using System;
using System.Security.Permissions;
using System.Web;
using System.Web.UI;
using System.ComponentModel;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace Buayacorp.Web
{
	[
		ToolboxData("<{0}:RatingControl runat=\"server\" />"),
		AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal),
		AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)
	]
	public class BCRating : WebControl, IPostBackEventHandler
	{
		#region Propiedades
		
		public bool UseCustomCss
		{
			get
			{
				if (ViewState["UseCustomCss"] != null)
				{
					return (bool)ViewState["UseCustomCss"];
				}
				return false;
			}
			set
			{
				ViewState["UseCustomCss"] = value;
			}
		}
		
		[Category("Appearance"), Description("Gets or sets the maximum allowed value."), DefaultValue(5)]
		public int SelectedRating
		{
			get
			{
				if (ViewState["SelectedRating"] != null)
				{
					return (int)ViewState["SelectedRating"];
				}
				return -1;
			}
			set
			{
				ViewState["SelectedRating"] = value;
			}
		}

		[Description("Gets or sets the current integer value."), Category("Appearance"), DefaultValue(0)]
		public double Value
		{
			get
			{
				if (ViewState["Value"] != null)
				{
					return (double)ViewState["Value"];
				}
				return 0;
			}
			set
			{
				ViewState["Value"] = value;
			}
		}
		#endregion

		#region "Renderizado" del control
		protected override void Render(HtmlTextWriter writer)
		{
			Page.VerifyRenderingInServerForm(this);

			if (!DesignMode)
			{
				RenderCss(writer);
				RenderLinks(writer);
			}
			else
				writer.Write(ID);
		}

		private void RenderCss(HtmlTextWriter writer)
		{
			if (HttpContext.Current.Items["rating_ctrl"] == null && !UseCustomCss) // Slo agregar un tag <link /> a la pgina
			{
				string resource = Page.ClientScript.GetWebResourceUrl(GetType(), "Buayacorp.Web.Resources.rating.css");

				HtmlLink lnk = new HtmlLink();
				lnk.Href = resource;
				lnk.Attributes["rel"] = "stylesheet";
				lnk.Attributes["type"] = "text/css";

				Controls.Add(lnk);
				lnk.RenderControl(writer);

				HttpContext.Current.Items["rating_ctrl"] = true;
			}
		}

		private void RenderLinks(HtmlTextWriter writer)
		{
			writer.AddAttribute(HtmlTextWriterAttribute.Class, "star-rating");
			writer.RenderBeginTag(HtmlTextWriterTag.Ul);

			int width = (int)Math.Floor(25 * Value);

			writer.AddAttribute(HtmlTextWriterAttribute.Class, "current-rating");
			writer.AddAttribute(HtmlTextWriterAttribute.Style, string.Format("width:{0}px", width));
			writer.RenderBeginTag(HtmlTextWriterTag.Li);
			writer.Write(string.Format("Actualmente {0}/5", Value));
			writer.RenderEndTag();

			links = new HtmlAnchor[5];
			for (int i = 0; i < links.Length; i++)
			{
				links[i] = new HtmlAnchor();
				links[i].EnableViewState = false;
				links[i].InnerText = (i + 1).ToString();
				links[i].HRef = Page.ClientScript.GetPostBackClientHyperlink(this, (i + 1).ToString());
				links[i].Attributes["class"] = "star" + (i + 1).ToString();
				links[i].Title = (i + 1) + " de " + links.Length;
				Controls.Add(links[i]);

				writer.RenderBeginTag(HtmlTextWriterTag.Li);
				links[i].RenderControl(writer);
				writer.RenderEndTag();
			}
			writer.RenderEndTag();
		} 
		#endregion
		
		public event EventHandler SelectedRatingChanged;
		
		private HtmlAnchor[] links;

		#region IPostBackEventHandler Members

		void IPostBackEventHandler.RaisePostBackEvent(string eventArgument)
		{
			if (!string.IsNullOrEmpty(eventArgument))
			{
				SelectedRating = int.Parse(eventArgument);

				if (SelectedRatingChanged != null)
					SelectedRatingChanged(this, EventArgs.Empty);
			}
		}

		#endregion
	}
}
